/*
 * Copyright (C) 2005 Sveinbjorn Thordarson
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software Foundation,
 * Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#define PZ_COMPAT
#include "pz.h"
#include <stdio.h>
//#include "oss.h"
#include <unistd.h>
#include <stdlib.h>
#include <fcntl.h>

void new_piano_window (void);
static int piano_handle_event(GR_EVENT *event);
void new_piano_window (void);
static void piano_do_draw (void);
static void piano_draw();
static void piano_loop();
static void piano_draw_arrow (int x, int y);
static void piano_draw_notename (int notenum);
static void piano_playnote (int notenum);
unsigned int read_32_len(int file_fd);

static GR_WINDOW_ID piano_wid;
static GR_GC_ID piano_gc_black;
static GR_GC_ID piano_gc_white;
static GR_GC_ID piano_gc_gray;

static GR_TIMER_ID piano_timer_id;


static pz_dsp_st dsppiano;

#define PIANO_HEIGHT	50
#define PIANO_WIDTH		118

#define PIANO_BLACKNOTE_HEIGHT	33
#define PIANO_NOTE_WIDTH		5

#define PIANO_NUMNOTES			24

#define PZ_DSP_PLAY_SIZE	4*1024*2

static unsigned short pcm_pianobuf[64*1024];

int piano_hascolor;

int piano_xoffset = 22;
int piano_yoffset = 20;

int piano_currnote = 0;
int piano_lastnote = 0;

char piano_notenames[24] = "CCDDEFFGGAABCCDDEFFGGAAB";
char piano_notesharp[24] = "010100101010010100101010";

typedef struct
{
	int x;
	int y;

} piano_point;

void new_piano_window (void)
{
	GrGetScreenInfo(&screen_info);

	if (screen_info.bpp > 2) // does we have a color display?
		piano_hascolor = 1;

	// location of piano depends on size of display
	piano_xoffset = (screen_info.cols - PIANO_WIDTH) / 2;

    piano_gc_black = pz_get_gc(1);   /* Get a copy of the root graphics context */
	/*Set foreground color */
    GrSetGCUseBackground(piano_gc_black, GR_FALSE);
    GrSetGCForeground(piano_gc_black, BLACK);
	
	piano_gc_black = pz_get_gc(1);
	GrSetGCUseBackground(piano_gc_black, GR_FALSE);
	GrSetGCForeground(piano_gc_black, BLACK);

	piano_gc_white = pz_get_gc(1);
	GrSetGCForeground(piano_gc_white, WHITE);
	
	piano_gc_gray = pz_get_gc(1);
	GrSetGCForeground(piano_gc_gray, GRAY);
	

    /* Open the window: */
    piano_wid = pz_new_window (0, HEADER_TOPLINE + 1, /* (x, y) */
                                  screen_info.cols,  
                                  screen_info.rows - (HEADER_TOPLINE + 1), /* Height of screen - header  */
                                  piano_do_draw, /* function pointer, called on an EXPOSURE event */
                                  piano_handle_event); /* function pointer, called on all other selected events */

    /* Select the types of events you need for your window: */
    GrSelectEvents (piano_wid, GR_EVENT_MASK_EXPOSURE | 
                      GR_EVENT_MASK_KEY_DOWN | GR_EVENT_MASK_KEY_UP
                    | GR_EVENT_MASK_TIMER);

    /* Map the window to the screen: */
    GrMapWindow (piano_wid);

    /* Create the timer used for animating your application: */
    piano_timer_id = GrCreateTimer (piano_wid,
                                       150); /* Timer interval, millisecs */
}

static void piano_do_draw (void)
{
    pz_draw_header("Piano");
    piano_draw(); /* the call to your draw function */
}

static int piano_handle_event(GR_EVENT *event)
{
    int ret = 0;
    static int paused = 0;

    switch (event->type) {
    case GR_EVENT_TYPE_TIMER:
        if (!paused)
            piano_loop();
        break;

    case GR_EVENT_TYPE_KEY_DOWN:
        switch (event->keystroke.ch) {
        case IPOD_WHEEL_ANTICLOCKWISE: /* Wheel left */
			break;
        case IPOD_WHEEL_CLOCKWISE: /* Wheel right */
            break;
        case IPOD_SWITCH_HOLD: /* hold switch on */
            paused = 1; /* pause the loop */
            break;
        case IPOD_BUTTON_MENU: /* Menu button */
            pz_close_window(piano_wid);
            GrDestroyTimer(piano_timer_id);
            GrDestroyGC(piano_gc_black);
			GrDestroyGC(piano_gc_white);
			GrDestroyGC(piano_gc_gray);
            break;
        case IPOD_BUTTON_ACTION: /* Action button */
			piano_playnote(piano_currnote);
			break;
        case IPOD_BUTTON_PLAY: /* Play/pause button */
        case IPOD_BUTTON_REWIND: /* Rewind button */
			piano_lastnote = piano_currnote;
			piano_currnote--;
			if (piano_currnote < 0)
				piano_currnote = PIANO_NUMNOTES-1;
			ret |= KEY_CLICK;
			break;
        case IPOD_BUTTON_FORWARD: /* Fast forward button */
			piano_lastnote = piano_currnote;
			piano_currnote++;
			if (piano_currnote > PIANO_NUMNOTES-1)
				piano_currnote = 0;
            ret |= KEY_CLICK; /* click on this event */
            break;
			break;
		default:
            ret |= KEY_UNUSED; /* allow a default key action */
            break;
        }
        break;
    case GR_EVENT_TYPE_KEY_UP:
        switch (event->keystroke.ch) {
        case IPOD_SWITCH_HOLD: /* un-held */
            paused = 0;
            break;
        default:
            ret |= KEY_UNUSED;
            break;
        }
    default:
        ret |= EVENT_UNUSED; /* we didn't ask for this event, go away */
    }
    return ret;
}


static void piano_draw()
{


    /* Clear the window */
    GrClearWindow(piano_wid, GR_FALSE);

//	GrLine(piano_wid, piano_gc_black, 22, 20, 22, 68);
//	GrLine(piano_wid, piano_gc_black, 140, 20, 140, 68);

	// center location depends on what kind of screen we have
	

	// draw frame around entire board
	GrRect(piano_wid, piano_gc_black, piano_xoffset, piano_yoffset, PIANO_WIDTH, PIANO_HEIGHT+1);
	
	// draw black notes
	GrFillRect(piano_wid, piano_gc_black, 5+piano_xoffset, piano_yoffset, PIANO_NOTE_WIDTH, PIANO_BLACKNOTE_HEIGHT); //x, y, width, height
	GrFillRect(piano_wid, piano_gc_black, 15+piano_xoffset, piano_yoffset, PIANO_NOTE_WIDTH, PIANO_BLACKNOTE_HEIGHT);
	GrFillRect(piano_wid, piano_gc_black, 30+piano_xoffset, piano_yoffset, PIANO_NOTE_WIDTH, PIANO_BLACKNOTE_HEIGHT);
	GrFillRect(piano_wid, piano_gc_black, 40+piano_xoffset, piano_yoffset, PIANO_NOTE_WIDTH, PIANO_BLACKNOTE_HEIGHT);
	GrFillRect(piano_wid, piano_gc_black, 50+piano_xoffset, piano_yoffset, PIANO_NOTE_WIDTH, PIANO_BLACKNOTE_HEIGHT);
	GrFillRect(piano_wid, piano_gc_black, 64+piano_xoffset, piano_yoffset, PIANO_NOTE_WIDTH, PIANO_BLACKNOTE_HEIGHT);
	GrFillRect(piano_wid, piano_gc_black, 74+piano_xoffset, piano_yoffset, PIANO_NOTE_WIDTH, PIANO_BLACKNOTE_HEIGHT);
	GrFillRect(piano_wid, piano_gc_black, 88+piano_xoffset, piano_yoffset, PIANO_NOTE_WIDTH, PIANO_BLACKNOTE_HEIGHT);
	GrFillRect(piano_wid, piano_gc_black, 98+piano_xoffset, piano_yoffset, PIANO_NOTE_WIDTH, PIANO_BLACKNOTE_HEIGHT);
	GrFillRect(piano_wid, piano_gc_black, 108+piano_xoffset, piano_yoffset, PIANO_NOTE_WIDTH, PIANO_BLACKNOTE_HEIGHT);
	
	// draw lines seperating white notes
	GrLine(piano_wid, piano_gc_black, 8+piano_xoffset, 33+piano_yoffset, 8+piano_xoffset, PIANO_HEIGHT+piano_yoffset); //x1,y1, x2,y2
	GrLine(piano_wid, piano_gc_black, 16+piano_xoffset, 33+piano_yoffset, 16+piano_xoffset, PIANO_HEIGHT+piano_yoffset);
	GrLine(piano_wid, piano_gc_black, 24+piano_xoffset, piano_yoffset, 24+piano_xoffset, PIANO_HEIGHT+piano_yoffset);
	GrLine(piano_wid, piano_gc_black, 33+piano_xoffset, 33+piano_yoffset, 33+piano_xoffset, PIANO_HEIGHT+piano_yoffset);
	GrLine(piano_wid, piano_gc_black, 42+piano_xoffset, 33+piano_yoffset, 42+piano_xoffset, PIANO_HEIGHT+piano_yoffset);
	GrLine(piano_wid, piano_gc_black, 51+piano_xoffset, 33+piano_yoffset, 51+piano_xoffset, PIANO_HEIGHT+piano_yoffset);
	GrLine(piano_wid, piano_gc_black, 59+piano_xoffset, piano_yoffset, 59+piano_xoffset, PIANO_HEIGHT+piano_yoffset);
	GrLine(piano_wid, piano_gc_black, 67+piano_xoffset, 33+piano_yoffset, 67+piano_xoffset, PIANO_HEIGHT+piano_yoffset);
	GrLine(piano_wid, piano_gc_black, 75+piano_xoffset, 33+piano_yoffset, 75+piano_xoffset, PIANO_HEIGHT+piano_yoffset);
	GrLine(piano_wid, piano_gc_black, 83+piano_xoffset, piano_yoffset, 83+piano_xoffset, PIANO_HEIGHT+piano_yoffset);
	GrLine(piano_wid, piano_gc_black, 91+piano_xoffset, 33+piano_yoffset, 91+piano_xoffset, PIANO_HEIGHT+piano_yoffset);
	GrLine(piano_wid, piano_gc_black, 100+piano_xoffset, 33+piano_yoffset, 100+piano_xoffset, PIANO_HEIGHT+piano_yoffset);
	GrLine(piano_wid, piano_gc_black, 110+piano_xoffset, 33+piano_yoffset, 110+piano_xoffset, PIANO_HEIGHT+piano_yoffset);

	piano_draw_arrow(piano_xoffset + (piano_currnote * PIANO_NOTE_WIDTH), piano_yoffset-7 );
	piano_draw_notename(piano_currnote);
    /* Draw the counter */
    /*sprintf(s, "Counter: %d", piano_counter);
    GrText(piano_wid,
           piano_gc,
           1, 103,  
           s,
           -1,
           GR_TFASCII);*/
}

static void piano_draw_arrow (int x, int y)
{
	GrLine(piano_wid, piano_gc_black, x+1, y, x+1, y+7);
	GrLine(piano_wid, piano_gc_black, x+2, y, x+2, y+8);
	GrLine(piano_wid, piano_gc_black, x+3, y, x+3, y+7);
	
	GrLine(piano_wid, piano_gc_black, x-1, y+4, x+6, y+4);
	GrLine(piano_wid, piano_gc_black, x, y+5, x+4, y+5);
}

static void piano_draw_notename (int notenum)
{
	char notestr[12];
	char sharp;
	
	if (piano_notesharp[notenum] == '1')
		sharp = '#';
	else
		sharp = ' ';

	sprintf(notestr, "Note: %c%c", piano_notenames[notenum], sharp);
	GrText(piano_wid, piano_gc_black, 2, screen_info.rows-25, notestr, -1, GR_TFASCII);
}

static void piano_playnote (int notenum)
{
	int		ret, count, samplerate;
	int		finished = 0;
	char	notefile[100];
	int		wavfile;
	char	sharp;
	
	//sharp = (piano_notesharp[notenum] == '1') ? sharp = '#' : sharp = 'a';
	sprintf(notefile, "/%c.wav", piano_notenames[notenum]);
	
	wavfile = open((char *)notefile, O_RDONLY);
	if (wavfile < NULL)
	{
		printf("Hello\n");
		return;
	}
	
	samplerate = read_32_len(wavfile);
	
	ret = pz_dsp_open(&dsppiano, PZ_DSP_LINEOUT);
	pz_dsp_setup(&dsppiano, 1, 44100);
	
	
	
	do
	{
		unsigned short *p = pcm_pianobuf;
		
		count = read(wavfile, (void *)p,
				PZ_DSP_PLAY_SIZE * (samplerate > 8000 ? 2 : 1));
		pz_dsp_write(&dsppiano, (void *)p, count);
		
		if (count < 1)
			finished = 1;
		
	} while (!finished);
	
	pz_dsp_close(&dsppiano);
}

static void piano_loop()
{
    /* Modify the state of your application: */
    

    /* Redraw the main window: */
    if (piano_currnote != piano_lastnote)
	{
		piano_draw(); 
		piano_lastnote = piano_currnote;
	}
}

unsigned int read_32_len(int file_fd)
{
	int value;

	read(file_fd, &value, 4);
#ifdef IS_BIG_ENDIAN
	value = bswap_32(value);
#endif

	return value;
}

PZ_SIMPLE_MOD ("piano", new_piano_window, "/Extras/Demos/Piano")
